# EnSimpleStaging  |  Copyright(C), 2004, Enfold Systems, LLC
# see LICENSE.txt for details

from Products.Archetypes.public import listTypes
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.permissions import ManagePortal
from Products.Archetypes.Extensions.utils import installTypes, install_subskin
from Products.ZCatalog.Catalog import CatalogError
from Products.ZCatalog.ProgressHandler import ZLogHandler

from Products.EnSimpleStaging.config import PROJECTNAME, GLOBALS
from Products.EnSimpleStaging.config import TOOLS, DEPENDENCIES
from Products.EnSimpleStaging.Extensions import Workflow

from StringIO import StringIO

PROPERTIES = {
    'always_display': (True, 'boolean'),
    }

CONTROL_PANEL = {
    'Simple Staging': dict(
    id='Simple Staging',
    name='Simple Staging',
    action='string:${portal_url}/prefs_simple_staging',
    condition='',
    permission=ManagePortal,
    category='Products',
    appId='Simple Staging',
    imageUrl='staging-area-icon.gif'),
    }

def setup_properties(self, out):
    pp = getToolByName(self, 'portal_properties')
    title = 'Simple Staging config'
    pname = 'simple_staging'
    props = getToolByName(pp, pname, None)
    if props is None:
        print >> out, 'Simple Staging settings did not exist, creating.'
        pp.addPropertySheet(pname, title=title)
        props = getToolByName(pp, pname)
    for pname, psettings in PROPERTIES.items():
        if props.hasProperty(pname):
            print >> out, "Property '%s' already exist, skipping." % pname
            continue
        print >> out, "Creating property '%s'." % pname
        props.manage_addProperty(pname, *psettings)
    print >> out, '%s setup successfully.' % title

def setup_site_config(self, out):
    cp = getToolByName(self, 'portal_controlpanel')
    existing = [c.id for c in cp._actions]
    for pname, psettings in CONTROL_PANEL.items():
        if not pname in existing:
            print >> out, "Configlet for '%s' did not exist, creating." % pname
            cp.registerConfiglet(**psettings)
    print >> out, "Site Config panel registered successfully"

def install_vc(self, out):
    if not hasattr(self, 'VersionRepository'):
        zvc = self.manage_addProduct['ZopeVersionControl']
        zvc.addRepository('VersionRepository')
        out.write("Created Version Repository.\n")

def install_tools(self, product, tools, out):
    addTool = self.manage_addProduct[product].manage_addTool
    for toolid, toolname in tools:
        # Don't remove tools, only re-add if necessary:
        if not hasattr(self, toolid):
            addTool(toolname)
            out.write("Created tool: %s (%s).\n" % (toolid, toolname))

def setup_local_tools(self, out, path='public_website'):
    from Acquisition import aq_parent, aq_base
    from Products.Archetypes.Extensions.utils import install_archetypetool
    from Products.Archetypes.Extensions.utils import install_uidcatalog
    from Products.Archetypes.Extensions.utils import install_templates
    from Products.Archetypes.Extensions.utils import install_referenceCatalog
    public = self.restrictedTraverse(path)
    # Hack around acquisition so that tools get setup correctly
    public = aq_base(public).__of__(aq_parent(self))
    aq_base(public).ESS_INSTALLING_LOCAL_TOOLS = True
    install_uidcatalog(public, out)
    install_referenceCatalog(public, out)
    install_archetypetool(public, out)
    install_templates(public, out)
    del public.ESS_INSTALLING_LOCAL_TOOLS
    out.write('Installed uid and reference catalogs on public_website.\n')

def setup_catalog(self, out=None):
    ct = getToolByName(self, 'portal_catalog')

    reindex = []
    msg = 'No indexes touched'

    if not 'isResourceChanged' in ct.schema():
        ct.manage_addColumn('isResourceChanged')

    if not 'isResourceChanged' in ct.indexes():
        ct.manage_addIndex('isResourceChanged', 'FieldIndex')
        reindex.append('isResourceChanged')
    
    if not 'lastModified' in ct.indexes():
        ct.manage_addIndex('lastModified', 'FieldIndex', 
                           extra={'indexed_attrs': 'modified'})
        reindex.append('lastModified')

    if reindex:
        msg = 'Created new indexes: %s' % ', '.join(reindex)
        handler = ZLogHandler(1000)
        ct.reindexIndex(reindex, self.REQUEST, handler)

    if out is not None:
        print >> out, msg
        return out.getvalue()
    return msg

def install_qi_products(self, out, products):
    qiTool = getToolByName(self, 'portal_quickinstaller')
    for product in products:
        qiTool.installProduct(product)
        print >> out, 'Installed dependency: %s' % product

def configure_kupu(self, out=None):
    zmi = False
    if out is None:
        zmi = True
        out = StringIO()
    tool = getToolByName(self, 'kupu_library_tool', None)
    if tool is not None:
        print >> out, 'Configuring kupu tool.'
        portal_types = tool.getPortalTypesForResourceType('collection')
        if 'StagingArea' not in portal_types:
            portal_types = list(portal_types)
            portal_types.append('StagingArea')
            tool.addResourceType('collection', portal_types)
    else:
        print >> out, 'Kupu tool not found skipping kupu config.'
    if zmi:
        return out.getvalue()

def install(self, reinstall=False):
    out = StringIO()

    install_qi_products(self, out, DEPENDENCIES)
    installTypes(self, out, listTypes(PROJECTNAME), PROJECTNAME)

    install_subskin(self, out, GLOBALS)
    install_vc(self, out)

    setup_catalog(self, out)
    setup_properties(self, out)
    setup_site_config(self, out)
    configure_kupu(self, out)
    Workflow.install(self, out)

    out.write("Successfully installed %s." % PROJECTNAME)
    return out.getvalue()

def afterInstall(self, reinstall, product):
    out = StringIO()
    if not reinstall:
        install_tools(self, PROJECTNAME, TOOLS, out)

def beforeUninstall(portal, reinstall, product, cascade):
    # Save ourselves from the quickinstaller
    ret = ''
    if 'portalobjects' in cascade:
        ret += 'Remove portalobjects from list of items to be uninstalled'
        cascade.remove('portalobjects')
    return ret, cascade
