# EnSimpleStaging  |  Copyright(C), 2004, 2008 Enfold Systems, LLC
# see LICENSE.txt for details

from zope.interface import implements, Interface, Attribute

class IInPlaceStaging(Interface):
    """Marker interface for in-place staging
    """

class IEvent(Interface):
    """Base Event Interface
    """

class IPublishStageStartEvent(IEvent):
    """ Deployment started
    """
    stage = Attribute('Stage being deployed')
    info = Attribute('Extra information about stage being deployed')

class IPublishStageFailureEvent(IEvent):
    """ Deployment finished
    """
    stage = Attribute('Stage being deployed')

class IPublishStageFinishEvent(IEvent):
    """ Deployment finished
    """
    stage = Attribute('Stage being deployed')

class IObjectStagedEvent(IEvent):
    """Notify the framework about the staging of an object.
    """

    source = Attribute('Source object being staged')
    target = Attribute('Target object being staged')
    full = Attribute('Is it part of a full or partial deployment?')
    stage = Attribute('The source stage')

class IObjectTaggedEvent(IEvent):
    """Notify the framework about the tagging of an object.
    """

    source = Attribute('Source object being tagged')
    stage = Attribute('The source stage')

class IObjectCheckInEvent(IEvent):
    """Notify the framework about the checkin of an object.
    """

    source = Attribute('Source object being checked in')
    stage = Attribute('The source stage')

class IExceptionLoggedEvent(IEvent):
    """Notify the framework that an exception happened and was logged
    """
    source = Attribute('The SiteErrorLog where this error was logged')
    entry_id = Attribute('The entry id for the logged exception on SiteErrorLog')
