# EnSimpleStaging  |  Copyright(C), 2004, Enfold Systems, LLC
# see LICENSE.txt for details

import os, sys
from sets import Set

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase
from Testing.ZopeTestCase.functional import Functional
from Products.PloneTestCase import PloneTestCase

for product in ('Archetypes',
                'PortalTransforms',
                'ZopeVersionControl',
                'CMFStaging',
                'EnSimpleStaging',
                'MimetypesRegistry',
#                'SecureMailHost',
                ):
    PloneTestCase.installProduct(product)
del product

from cStringIO import StringIO
from DateTime import DateTime
from Products.CMFCore.utils import getToolByName
from Products.Archetypes.tests.utils import makeContent
from Products.EnSimpleStaging.content.stage import DeploymentError
from Products.EnSimpleStaging.tests.base import TestStaging
from Products.EnSimpleStaging.tests.base import TestZopeStaging
from Products.EnSimpleStaging.tests.demostorage import StorageStackingTestCase
from Products.EnSimpleStaging.tools.workspaces import StagingError

PloneTestCase.setupPloneSite()

class TestChangeSets(StorageStackingTestCase):

    def afterSetUp(self):
        TestStaging.afterSetUp(self)
        self.setupStagingAreas()

    def testSimpleTaggingIsReversible(self):
        mkt = self.home.mkt
        public = self.public
        self.makeContent(public, 'Folder', id='marketing')
        mkt.setStagePath('public_website/marketing')
        self.makeContent(mkt, 'Folder', id='afolder')
        mkt.saveStage(label='initial')
        index, annotLast = mkt.getLastReversibleAnnotation()

    def testChangeSetsFullDeployment(self):
        mkt = self.home.mkt
        public = self.public
        self.makeContent(public, 'Folder', id='marketing')
        mkt.setStagePath('public_website/marketing')

        getChanges = mkt.getLastMovementsAndModifications

        # 0. Must start by making at least a tag.
        mkt.saveStage(label='initial')

        # 1. Create a document, publish stage, with a label. Should
        # catch the add for this document.
        self.makeContent(mkt, 'Document', id='doc1')
        mkt.publishStage(label='1')
        self.commit()

        cs = getChanges()
        self.assertEquals(cs['adds'], [('doc1',)])

        # 2. Create a folder, edit existing document, publish stage,
        # with a label. Should catch the folder's add and the document
        # modification.
        self.makeContent(mkt, 'Folder', id='folder1')
        self.setTitle(mkt.doc1, 'Document 1')

        mkt.publishStage(label='2')
        self.commit()

        cs = getChanges()
        self.assertEquals(cs['adds'], [('folder1',)])
        self.assertEquals(cs['modifications'], [('doc1',)])

        # 3. Create document inside folder, remove doc1, publish stage
        # with a label. Should catch the new document's add, and the
        # old document remove plus a folder1 change.
        self.makeContent(mkt.folder1, 'Document', id='doc11')
        mkt.manage_delObjects(ids=['doc1'])
        mkt.publishStage(label='3')
        self.commit()

        cs = getChanges()
        self.assertEquals(cs['adds'], [('folder1', 'doc11')])
        self.assertEquals(cs['modifications'], [('folder1',)])
        self.assertEquals(cs['removes'], [('doc1',)])

        # 4. Remove folder, add a new folder with the same name,
        # publish stage with a label. Need to catch that the new
        # folder is different from the old one, thus yielding a remove
        # for the folder1 and the contained doc11, and a add for
        # folder1 and doc2.
        mkt.manage_delObjects(ids=['folder1'])
        self.makeContent(mkt, 'Folder', id='folder1')
        self.makeContent(mkt, 'Document', id='doc2')
        mkt.publishStage(label='4')
        self.commit()

        cs = getChanges()
        self.assertEquals(cs['adds'], [('doc2',), ('folder1',)])
        self.assertEquals(cs['removes'], [('folder1',), ('folder1', 'doc11')])

    def testChangeSetsPartialDeployment(self):
        mkt = self.home.mkt
        public = self.public
        self.makeContent(public, 'Folder', id='marketing')
        mkt.setStagePath('public_website/marketing')

        getChanges = mkt.getLastMovementsAndModifications

        # 0. Must start by making at least a tag.
        mkt.saveStage(label='initial')

        # 1. Create a document, publish stage, with a label. Should
        # catch the add for this document.
        self.makeContent(mkt, 'Document', id='doc1')
        mkt.publishStage(label='1')
        self.commit()

        cs = getChanges()
        self.assertEquals(cs['adds'], [('doc1',)])

        # 2. Create a folder, edit existing document, publish stage,
        # with a label. Should catch the folder's add and the document
        # modification.
        self.makeContent(mkt, 'Folder', id='folder1')
        self.setTitle(mkt.doc1, 'Document 1')
        # doc is a CMFDefault document. doc.setTitle() doesn't trigger a
        # reindex
        self.touchContent(mkt.doc1)
        self.commit()

        mkt.publishLastChanged(label='2')
        self.commit()

        cs = getChanges()
        self.assertEquals(cs['adds'], [('folder1',)])
        # Should work since we reindexed doc1 above.
        self.assertEquals(cs['modifications'], [('doc1',)])

        # 3. Create document inside folder, remove doc1, publish stage
        # with a label. Should catch the new document's add, and the
        # old document remove plus a folder1 change.
        self.makeContent(mkt.folder1, 'Document', id='doc11')
        mkt.manage_delObjects(ids=['doc1'])
        self.commit()

        mkt.publishLastChanged(label='3')

        cs = getChanges()
        self.assertEquals(cs['adds'], [('folder1', 'doc11')])
        # Next one should work if the patch to _setOb got applied.
        self.assertEquals(cs['modifications'], [('folder1',)])
        self.assertEquals(cs['removes'], [('doc1',)])

        # 4. Remove folder, add a new folder with the same name,
        # publish stage with a label. Need to catch that the new
        # folder is different from the old one, thus yielding a remove
        # for the folder1 and the contained doc11, and a add for
        # folder1 and doc2.
        mkt.manage_delObjects(ids=['folder1'])
        self.makeContent(mkt, 'Folder', id='folder1')
        self.makeContent(mkt, 'Document', id='doc2')
        self.commit()

        if not mkt.getStagingUseCatalogOnly():
            removes, adds = mkt.getLastAddedAndRemoved()
            removes = [p[len('/'.join(mkt.getPhysicalPath()))+1:] for p in removes]
            adds = [p[len('/'.join(mkt.getPhysicalPath()))+1:] for p in adds]

            self.assertEquals(removes, ['folder1', 'folder1/doc11'])
            self.assertEquals(adds, ['doc2', 'folder1'])

            changed = mkt.getLastChangedPaths()
            changed = [p[len('/'.join(mkt.getPhysicalPath()))+1:] for p in changed]
            self.assertEquals(changed, [])

        mkt.publishLastChanged(label='4')

        cs = getChanges()
        self.assertEquals(cs['adds'], [('doc2',), ('folder1',)])
        self.assertEquals(cs['removes'], [('folder1',), ('folder1', 'doc11')])

    def testLastMovements(self):
        mkt = self.home.mkt
        mkt.setStagePath('public_website')

        # Publish empty stage. Need a initial tag.
        mkt.saveStage(label='initial')
        mkt.publishStage()
        self.commit()

        getChanges = mkt.getLastMovementsAndModifications

        # Everything should be empty.
        info = getChanges()
        self.failIf(info['adds'])
        self.failIf(info['modifications'])
        self.failIf(info['removes'])

        # Added f1, f1/d11 and d1
        self.makeContent(mkt, 'Folder', 'f1')
        self.makeContent(mkt.f1, 'Document', 'd11', title='Document One')
        self.makeContent(mkt, 'Document', 'd1')

        if not mkt.getStagingUseCatalogOnly():
            removes, adds = mkt.getLastAddedAndRemoved()
            removes = [p[len('/'.join(mkt.getPhysicalPath()))+1:] for p in removes]
            adds = [p[len('/'.join(mkt.getPhysicalPath()))+1:] for p in adds]

            self.assertEquals(removes, [])
            self.assertEquals(adds, ['d1', 'f1', 'f1/d11'])

            changed = mkt.getLastChangedPaths()
            changed = [p[len('/'.join(mkt.getPhysicalPath()))+1:] for p in changed]
            self.assertEquals(changed, [])

        mkt.publishLastChanged()
        self.commit()

        info = getChanges()
        self.assertEquals(info['adds'],
                          [('d1',), ('f1',), ('f1', 'd11')])
        self.assertEquals(info['modifications'], [])
        self.assertEquals(info['removes'], [])

        # Added f2, f2/d21 and removed d1
        self.makeContent(mkt, 'Folder', 'f2')
        self.makeContent(mkt.f2, 'Document', 'd21')
        mkt.manage_delObjects(ids=['d1'])

        if not mkt.getStagingUseCatalogOnly():
            changed = mkt.getLastChangedPaths()
            changed = [p[len('/'.join(mkt.getPhysicalPath()))+1:] for p in changed]
            self.assertEquals(changed, [])

        mkt.publishLastChanged()
        self.commit()

        info = getChanges()
        self.assertEquals(info['adds'],
                          [('f2',), ('f2', 'd21')])
        self.assertEquals(info['modifications'], [])
        self.assertEquals(info['removes'], [('d1',)])

        # Renamed 'd21' to 'd21a', equals to remove 'd21' and add 'd21a'
        self.commit()
        mkt.f2.manage_renameObjects(ids=['d21'], new_ids=['d21a'])
        self.commit()

        mkt.publishLastChanged()
        self.commit()

        info = getChanges()
        self.assertEquals(info['adds'],
                          [('f2', 'd21a')])
        self.assertEquals(info['modifications'], [('f2',)])
        self.assertEquals(info['removes'], [('f2', 'd21',)])

        # Renamed 'f2' to 'f2a', equals to remove f2, f2/d21a and add
        # f2a and f2a/d21a
        self.commit()
        mkt.manage_renameObjects(ids=['f2'], new_ids=['f2a'])
        self.commit()

        mkt.publishLastChanged()
        self.commit()

        info = getChanges()
        self.assertEquals(info['adds'],
                          [('f2a',), ('f2a', 'd21a')])
        self.assertEquals(info['modifications'], [])
        self.assertEquals(info['removes'], [('f2',), ('f2', 'd21a',)])

        # Make a minor modification to make sure it survives a publish.
        self.assertEquals(mkt.f1.d11.title, 'Document One')
        self.setTitle(mkt.f1.d11, 'Document Alpha One')
        self.assertEquals(mkt.f1.d11.title, 'Document Alpha One')
        self.commit()

        # Full publish, mark everything as modified
        mkt.publishStage()
        self.commit()

        info = getChanges()
        self.assertEquals(info['adds'], [])
        self.assertEquals(info['modifications'],
                          [('f1',), ('f1', 'd11'),
                           ('f2a',), ('f2a', 'd21a')])
        self.assertEquals(info['removes'], [])
        self.assertEquals(mkt.f1.d11.title, 'Document Alpha One')

    def testPublishSelectedChanges(self):
        mkt = self.home.mkt
        mkt.setStagePath('public_website')

        # Publish empty stage. Need a initial tag.
        mkt.saveStage(label='initial')
        mkt.publishStage()
        self.commit()

        getChanges = mkt.getLastMovementsAndModifications

        # Everything should be empty.
        info = getChanges()
        self.failIf(info['adds'])
        self.failIf(info['modifications'])
        self.failIf(info['removes'])

        # Added f1, f1/d11, d1 and d2
        self.makeContent(mkt, 'Folder', 'f1')
        self.makeContent(mkt.f1, 'Document', 'd11', title='Document One')
        self.makeContent(mkt, 'Document', 'd1')
        self.makeContent(mkt, 'Document', 'd2')
        self.commit()

        # Verify that the set of pending changes matches our expectations.
        removes, adds = mkt.getLastAddedAndRemoved()
        removes = [p[len('/'.join(mkt.getPhysicalPath()))+1:] for p in removes]
        adds = [p[len('/'.join(mkt.getPhysicalPath()))+1:] for p in adds]

        self.assertEquals(removes, [])
        self.assertEquals(adds, ['d1', 'd2', 'f1', 'f1/d11'])

        changed = mkt.getLastChangedPaths()
        changed = [p[len('/'.join(mkt.getPhysicalPath()))+1:] 
                   for p in changed]
        changed = list(set(changed) - set(adds))
        self.assertEquals(changed, [])

        # Publish only a subset of the pending changes
        mkt.publishSelectedChanges(adds=('d1',))
        self.commit()

        info = getChanges()
        self.assertEquals(info['adds'], [('d1',)])
        self.assertEquals(info['modifications'], [])
        self.assertEquals(info['removes'], [])

        # Verify that the set of pending changes matches expectations
        removes, adds = mkt.getLastAddedAndRemoved()
        removes = [p[len('/'.join(mkt.getPhysicalPath()))+1:] for p in removes]
        adds = [p[len('/'.join(mkt.getPhysicalPath()))+1:] for p in adds]

        self.assertEquals(removes, [])
        self.assertEquals(adds, ['d2', 'f1', 'f1/d11'])

        changed = mkt.getLastChangedPaths()
        changed = [p[len('/'.join(mkt.getPhysicalPath()))+1:] for p in changed]
        changed = list(set(changed) - set(adds))
        self.assertEquals(changed, [])

        # Publish only a subset of the pending changes
        mkt.publishSelectedChanges(adds=('f1', 'f1/d11',))
        self.commit()

        info = getChanges()
        self.assertEquals(info['adds'], [('f1',), ('f1', 'd11',)])
        self.assertEquals(info['modifications'], [])
        self.assertEquals(info['removes'], [])

        # Verify that pending changes match expectations
        removes, adds = mkt.getLastAddedAndRemoved()
        removes = [p[len('/'.join(mkt.getPhysicalPath()))+1:] for p in removes]
        adds = [p[len('/'.join(mkt.getPhysicalPath()))+1:] for p in adds]

        self.assertEquals(removes, [])
        self.assertEquals(adds, ['d2'])
        
        changed = mkt.getLastChangedPaths(applyFilter=True)
        changed = [p[len('/'.join(mkt.getPhysicalPath()))+1:] for p in changed]
        changed = list(set(changed) - set(adds))
        self.assertEquals(changed, [])

        # Publish *all* pending changes
        mkt.publishLastChanged()
        self.commit()

        info = getChanges()
        self.assertEquals(info['adds'], [('d2',)])
        self.assertEquals(info['modifications'], [])
        self.assertEquals(info['removes'], [])

        # Added f2, f2/d21 and removed d1
        self.makeContent(mkt, 'Folder', 'f2')
        self.makeContent(mkt.f2, 'Document', 'd21')
        mkt.manage_delObjects(ids=['d1'])

        if not mkt.getStagingUseCatalogOnly():
            changed = mkt.getLastChangedPaths()
            changed = [p[len('/'.join(mkt.getPhysicalPath()))+1:] for p in changed]
            self.assertEquals(changed, [])

        mkt.publishSelectedChanges(removes=('d1',))
        self.commit()

        info = getChanges()
        self.assertEquals(info['adds'], [])
        self.assertEquals(info['modifications'], [])
        self.assertEquals(info['removes'], [('d1',)])

        mkt.publishLastChanged()
        self.commit()

        info = getChanges()
        self.assertEquals(info['adds'],
                          [('f2',), ('f2', 'd21')])
        self.assertEquals(info['modifications'], [])
        self.assertEquals(info['removes'], [])

class TestWFChangeSets(TestStaging):

    def afterSetUp(self):
        TestStaging.afterSetUp(self)
        self.setupStagingAreas()

    def testWFStateChangeSetsFullDeployment(self):
        mkt = self.home.mkt
        public = self.public
        self.makeContent(public, 'Folder', id='marketing')
        mkt.setStagePath('public_website/marketing')
        mkt.setDeployStates(('published',))
        doInPlace = mkt.doInPlace()

        wf = getToolByName(mkt, 'portal_workflow')
        getChanges = mkt.getLastMovementsAndModifications
        def status(ob):
            return wf.getInfoFor(ob, 'review_state', default=None)

        def action(ob, action):
            return wf.doActionFor(ob, action)

        # 0. Must start by making at least a tag.
        mkt.saveStage(label='initial')

        # 1. Create a document, publish stage, with a label. Should
        # not catch the add for this document because it isn't in
        # 'published' state.
        self.makeContent(mkt, 'Document', id='doc1')
        self.assertNotEquals(status(mkt.doc1), 'published')

        mkt.publishStage(label='1')
        cs = getChanges()
        self.assertEquals(cs['adds'], [])

        # 2. Now, publish the document and then publish the stage. It
        # should now catch the add for this document.
        action(mkt.doc1, 'publish')
        self.assertEquals(status(mkt.doc1), 'published')

        mkt.publishStage(label='2')
        cs = getChanges()
        self.assertEquals(cs['adds'], [('doc1',)])

        # 3. Create a folder, edit existing document, publish stage,
        # with a label. Should not catch the folder's add because it's
        # not published, but should catch the document modification.
        self.makeContent(mkt, 'Folder', id='folder1')
        self.assertNotEquals(status(mkt.folder1), 'published')

        self.setTitle(mkt.doc1, 'Document 1')
        self.assertEquals(status(mkt.doc1), 'published')

        mkt.publishStage(label='3')

        cs = getChanges()
        self.assertEquals(cs['adds'], [])
        self.assertEquals(cs['modifications'], [('doc1',)])

        # 4. Create document inside folder, publish it publish stage
        # with a label. Should catch the new document's add and raise
        # an exception because the parent folder is not published
        self.makeContent(mkt.folder1, 'Document', id='doc11')
        action(mkt.folder1.doc11, 'publish')
        self.assertEquals(status(mkt.folder1.doc11), 'published')

        mkt.publishStage(label='4')
        
        cs = getChanges()
        self.assertEquals(cs['adds'], [])

        # 5. Now, publish containing folder and switch doc1 to not
        # being in 'published' state by doing a 'retract'. Should
        # catch the add of folder1 and doc11 and removal of doc1.
        action(mkt.folder1, 'publish')
        self.assertEquals(status(mkt.folder1), 'published')

        action(mkt.doc1, 'retract')
        self.assertNotEquals(status(mkt.doc1), 'published')

        mkt.publishStage(label='5')

        cs = getChanges()
        self.assertEquals(cs['adds'], [('folder1',), ('folder1', 'doc11')])
        self.assertEquals(cs['modifications'], [])
        self.assertEquals(cs['removes'], [('doc1',)])

        # 6. Remove folder, add a new folder with the same name,
        # publish stage with a label. Need to catch that the new
        # folder is different from the old one, thus yielding a remove
        # for the folder1 and the contained doc11, and a add for
        # folder1 and doc2.
        mkt.manage_delObjects(ids=['folder1'])

        self.makeContent(mkt, 'Folder', id='folder1')
        action(mkt.folder1, 'publish')
        self.assertEquals(status(mkt.folder1), 'published')

        self.makeContent(mkt, 'Document', id='doc2')
        action(mkt.doc2, 'publish')
        self.assertEquals(status(mkt.doc2), 'published')

        mkt.publishStage(label='6')

        cs = getChanges()
        self.assertEquals(cs['adds'], [('doc2',), ('folder1',)])
        self.assertEquals(cs['removes'], [('folder1',), ('folder1', 'doc11')])

        # 7. Add a new document into folder1, should catch the add of
        # this doc only.
        self.makeContent(mkt.folder1, 'Document', id='doc12')
        action(mkt.folder1.doc12, 'publish')
        self.assertEquals(status(mkt.folder1.doc12), 'published')

        mkt.publishStage(label='7')

        cs = getChanges()
        self.assertEquals(cs['adds'], [('folder1', 'doc12')])

        # 8. Change the state of the containing folder to not be
        # 'published'. Contained objects will not be deployed.
        action(mkt.folder1, 'retract')
        self.assertNotEquals(status(mkt.folder1), 'published')

        mkt.publishStage(label='8')
        
        cs = getChanges()
        self.assertEquals(cs['adds'], [])
        self.assertEquals(cs['removes'],
                          [('folder1',), ('folder1', 'doc12')])
        self.failIf('folder1' in public.marketing.objectIds())

    def testWFStateChangeSetsPartialDeployment(self):
        mkt = self.home.mkt
        public = self.public
        self.makeContent(public, 'Folder', id='marketing')
        mkt.setStagePath('public_website/marketing')
        mkt.setDeployStates(('published',))
        doInPlace = mkt.doInPlace()

        wf = getToolByName(mkt, 'portal_workflow')
        getChanges = mkt.getLastMovementsAndModifications
        def status(ob):
            return wf.getInfoFor(ob, 'review_state', default=None)

        def action(ob, action):
            return wf.doActionFor(ob, action)

        # 0. Must start by making at least a tag.
        mkt.saveStage(label='initial')

        # 1. Create a document, publish stage, with a label. Should
        # not catch the add for this document because it isn't in
        # 'published' state.
        self.makeContent(mkt, 'Document', id='doc1')
        self.assertNotEquals(status(mkt.doc1), 'published')

        # Verify that the set of pending changes matches our expectations.
        removes, adds = mkt.getLastAddedAndRemoved()
        removes = [p[len('/'.join(mkt.getPhysicalPath()))+1:] for p in removes]
        adds = [p[len('/'.join(mkt.getPhysicalPath()))+1:] for p in adds]

        self.assertEquals(removes, [])
        self.assertEquals(adds, [])

        # Since nothing is marked as changed, publishing raises an
        # exception now.
        self.assertRaises(DeploymentError, mkt.publishLastChanged, label=1)

        # 2. Now, publish the document and then publish the stage. It
        # should now catch the add for this document.
        action(mkt.doc1, 'publish')
        self.assertEquals(status(mkt.doc1), 'published')

        # Verify that the set of pending changes matches our expectations.
        removes, adds = mkt.getLastAddedAndRemoved()
        removes = [p[len('/'.join(mkt.getPhysicalPath()))+1:] for p in removes]
        adds = [p[len('/'.join(mkt.getPhysicalPath()))+1:] for p in adds]

        self.assertEquals(removes, [])
        self.assertEquals(adds, ['doc1'])

        # ENTRANSIT-116: Mark this content as expired by setting
        # 'expiration date' to the past. It should *not* show up in
        # the list of pending changes, even though the workflow state
        # says it should.

        mkt.doc1.setEffectiveDate(DateTime() - 2)
        mkt.doc1.setExpirationDate(DateTime() - 1)
        self.touchContent(mkt.doc1)
        self.commit()

        # Verify that the set of pending changes matches our expectations.
        removes, adds = mkt.getLastAddedAndRemoved()
        removes = [p[len('/'.join(mkt.getPhysicalPath()))+1:] for p in removes]
        adds = [p[len('/'.join(mkt.getPhysicalPath()))+1:] for p in adds]

        self.assertEquals(removes, [])
        self.assertEquals(adds, [])

        # Now set the expiration date back to a time in the future,
        # and it should show up in pending changes again.
        mkt.doc1.setExpirationDate(DateTime() + 1)
        self.touchContent(mkt.doc1)
        self.commit()

        # Verify that the set of pending changes matches our expectations.
        removes, adds = mkt.getLastAddedAndRemoved()
        removes = [p[len('/'.join(mkt.getPhysicalPath()))+1:] for p in removes]
        adds = [p[len('/'.join(mkt.getPhysicalPath()))+1:] for p in adds]

        self.assertEquals(removes, [])
        self.assertEquals(adds, ['doc1'])

        mkt.publishLastChanged(label='2')
        cs = getChanges()
        self.assertEquals(cs['adds'], [('doc1',)])

        # 3. Create a folder, edit existing document, publish stage,
        # with a label. Should not catch the folder's add because it's
        # not published, but should catch the document modification.
        self.makeContent(mkt, 'Folder', id='folder1')
        self.assertNotEquals(status(mkt.folder1), 'published')

        self.setTitle(mkt.doc1, 'Document 1')
        # doc is a CMFDefault document. doc.setTitle() doesn't trigger a
        # reindex
        self.touchContent(mkt.doc1)
        self.commit()

        self.assertEquals(status(mkt.doc1), 'published')

        # Verify that the set of pending changes matches our expectations.
        removes, adds = mkt.getLastAddedAndRemoved()
        removes = [p[len('/'.join(mkt.getPhysicalPath()))+1:] for p in removes]
        adds = [p[len('/'.join(mkt.getPhysicalPath()))+1:] for p in adds]

        self.assertEquals(removes, [])
        self.assertEquals(adds, [])

        mkt.publishLastChanged(label='3')
        self.commit()

        cs = getChanges()
        self.assertEquals(cs['adds'], [])
        # Should work since we reindexed doc1 above.
        self.assertEquals(cs['modifications'], [('doc1',)])

        # 4. Create document inside folder, publish it, publish stage
        # with a label. Should catch the new document's add and raise
        # an exception because the parent folder is not published
        self.makeContent(mkt.folder1, 'Document', id='doc11')
        action(mkt.folder1.doc11, 'publish')
        self.assertEquals(status(mkt.folder1.doc11), 'published')

        if not doInPlace:
            # If not an inplace deployment, a StagingError happens
            # because the containers are checked.
            self.commit()
            self.assertRaises(StagingError, mkt.publishLastChanged, label='4')
            self.abort()
        else:
            # in-place deployment doesn't check containers, because
            # obviously there are none to check. it won't find the add
            # of the subdocument though, because it won't be able to
            # find info about the parent folder.
            mkt.publishLastChanged(label='4')

            cs = getChanges()
            self.assertEquals(cs['adds'], [])

        # 5. Now, publish containing folder and switch doc1 to not
        # being in 'published' state by doing a 'retract'. Should
        # catch the add of folder1 and doc11 and removal of doc1.
        action(mkt.folder1, 'publish')
        self.assertEquals(status(mkt.folder1), 'published')

        action(mkt.doc1, 'retract')
        self.assertNotEquals(status(mkt.doc1), 'published')

        # ENTRANSIT-116: Mark folder1 content as expired by setting
        # 'expiration date' to the past. It should *not* show up in
        # the list of pending changes, even though the workflow state
        # says it should.
        #
        # Create another subfolder inside folder1, with yet another
        # document inside.
        self.makeContent(mkt.folder1, 'Folder', id='folder11')
        self.makeContent(mkt.folder1.folder11, 'Document', id='doc111')
        action(mkt.folder1.folder11, 'publish')
        action(mkt.folder1.folder11.doc111, 'publish')
        self.commit()

        # Verify that the set of pending changes matches our expectations.
        removes, adds = mkt.getLastAddedAndRemoved()
        removes = [p[len('/'.join(mkt.getPhysicalPath()))+1:] for p in removes]
        adds = [p[len('/'.join(mkt.getPhysicalPath()))+1:] for p in adds]

        self.assertEquals(removes, ['doc1'])
        self.assertEquals(adds, ['folder1', 
                                 'folder1/doc11', 
                                 'folder1/folder11',
                                 'folder1/folder11/doc111'])

        mkt.folder1.setEffectiveDate(DateTime() - 2)
        mkt.folder1.setExpirationDate(DateTime() - 1)
        self.touchContent(mkt.folder1)
        self.commit()

        # Verify that the set of pending changes matches our expectations.
        removes, adds = mkt.getLastAddedAndRemoved()
        removes = [p[len('/'.join(mkt.getPhysicalPath()))+1:] for p in removes]
        adds = [p[len('/'.join(mkt.getPhysicalPath()))+1:] for p in adds]

        self.assertEquals(removes, ['doc1'])
        self.assertEquals(adds, [])

        # Now set the expiration date back to a time in the future,
        # and it should show up in pending changes again. We also
        # delete the subfolder. It was there just for testing the
        # pending changes.
        mkt.folder1.setExpirationDate(DateTime() + 1)
        mkt.folder1.manage_delObjects(ids=['folder11'])
        self.touchContent(mkt.folder1)
        self.commit()

        # Verify that the set of pending changes matches our expectations.
        removes, adds = mkt.getLastAddedAndRemoved()
        removes = [p[len('/'.join(mkt.getPhysicalPath()))+1:] for p in removes]
        adds = [p[len('/'.join(mkt.getPhysicalPath()))+1:] for p in adds]

        self.assertEquals(removes, ['doc1'])
        self.assertEquals(adds, ['folder1', 'folder1/doc11'])

        mkt.publishLastChanged(label='5')

        cs = getChanges()
        self.assertEquals(cs['adds'], [('folder1',), ('folder1', 'doc11')])
        self.assertEquals(cs['modifications'], [])
        self.assertEquals(cs['removes'], [('doc1',)])

        # Verify that the set of pending changes matches our
        # expectations after the publication.
        removes, adds = mkt.getLastAddedAndRemoved()
        removes = [p[len('/'.join(mkt.getPhysicalPath()))+1:] for p in removes]
        adds = [p[len('/'.join(mkt.getPhysicalPath()))+1:] for p in adds]

        self.assertEquals(removes, [])
        self.assertEquals(adds, [])

        # ENTRANSIT-116: Mark folder1 content as expired. This time,
        # the folder already exists in the destination. It should show
        # up in the list of pending changes as pending removal and so
        # should the contained document.

        mkt.folder1.setEffectiveDate(DateTime() - 2)
        mkt.folder1.setExpirationDate(DateTime() - 1)
        self.touchContent(mkt.folder1)
        self.commit()

        # Verify that the set of pending changes matches our expectations.
        removes, adds = mkt.getLastAddedAndRemoved()
        removes = [p[len('/'.join(mkt.getPhysicalPath()))+1:] for p in removes]
        adds = [p[len('/'.join(mkt.getPhysicalPath()))+1:] for p in adds]

        self.assertEquals(removes, ['folder1', 'folder1/doc11'])
        self.assertEquals(adds, [])

        # Now set the expiration date back to a time in the future,
        # and it should show up in pending changes again.
        mkt.folder1.setExpirationDate(DateTime() + 1)
        self.touchContent(mkt.folder1)
        self.commit()

        # Verify that the set of pending changes matches our expectations.
        removes, adds = mkt.getLastAddedAndRemoved()
        removes = [p[len('/'.join(mkt.getPhysicalPath()))+1:] for p in removes]
        adds = [p[len('/'.join(mkt.getPhysicalPath()))+1:] for p in adds]

        self.assertEquals(removes, [])
        self.assertEquals(adds, [])

        # 6. Remove folder, add a new folder with the same name,
        # publish stage with a label. Need to catch that the new
        # folder is different from the old one, thus yielding a remove
        # for the folder1 and the contained doc11, and an add for
        # folder1 and doc2.
        mkt.manage_delObjects(ids=['folder1'])

        self.makeContent(mkt, 'Folder', id='folder1')
        action(mkt.folder1, 'publish')
        self.assertEquals(status(mkt.folder1), 'published')

        self.makeContent(mkt, 'Document', id='doc2')
        action(mkt.doc2, 'publish')
        self.assertEquals(status(mkt.doc2), 'published')

        mkt.publishLastChanged(label='6')

        cs = getChanges()
        self.assertEquals(cs['adds'], [('doc2',), ('folder1',)])
        self.assertEquals(cs['removes'], [('folder1',), ('folder1', 'doc11')])

        # 7. Add a new document into folder1, should catch the add of
        # this doc only.
        self.makeContent(mkt.folder1, 'Document', id='doc12')
        action(mkt.folder1.doc12, 'publish')
        self.assertEquals(status(mkt.folder1.doc12), 'published')

        mkt.publishLastChanged(label='7')

        cs = getChanges()
        self.assertEquals(cs['adds'], [('folder1', 'doc12')])

        # 8. Change the state of the containing folder to not be
        # 'published'. Contents should not be deployed.
        action(mkt.folder1, 'retract')
        self.assertNotEquals(status(mkt.folder1), 'published')

        # should find a removal for folder1 and doc12. what happens
        # here is that folder1 is deleted immediately from the target.
        mkt.publishLastChanged(label='8')

        cs = getChanges()
        self.assertEquals(cs['adds'], [])
        self.assertEquals(cs['removes'],
                          [('folder1',), ('folder1', 'doc12')])
        self.failIf('folder1' in public.marketing.objectIds())

    def testWFStateLastMovements(self):
        mkt = self.home.mkt
        mkt.setStagePath('public_website')
        mkt.setDeployStates(('published',))
        doInPlace = mkt.doInPlace()

        wf = getToolByName(mkt, 'portal_workflow')
        getChanges = mkt.getLastMovementsAndModifications
        def status(ob):
            return wf.getInfoFor(ob, 'review_state', default=None)

        def action(ob, action):
            return wf.doActionFor(ob, action)

        # Publish empty stage. Need a initial tag.
        mkt.saveStage(label='initial')
        mkt.publishStage()

        getChanges = mkt.getLastMovementsAndModifications

        # Everything should be empty.
        info = getChanges()
        self.failIf(info['adds'])
        self.failIf(info['modifications'])
        self.failIf(info['removes'])

        # Added f1, f1/d11 and d1
        self.makeContent(mkt, 'Folder', 'f1')
        self.makeContent(mkt.f1, 'Document', 'd11', title='Document One')
        self.makeContent(mkt, 'Document', 'd1')

        # Since no object is pending deployment, publishLastChanged
        # will raise an exception.
        self.assertRaises(DeploymentError, mkt.publishLastChanged)

        # Publish the objects
        action(mkt.f1, 'publish')
        action(mkt.f1.d11, 'publish')
        action(mkt.d1, 'publish')
        mkt.publishLastChanged()

        # Should see the changes now, all of the objects were in
        # 'published' state.
        info = getChanges()
        self.assertEquals(info['adds'],
                          [('d1',), ('f1',), ('f1', 'd11')])
        self.assertEquals(info['modifications'], [])
        self.assertEquals(info['removes'], [])

        # Added f2, f2/d21 and removed d1
        self.makeContent(mkt, 'Folder', 'f2')
        self.makeContent(mkt.f2, 'Document', 'd21')
        mkt.manage_delObjects(ids=['d1'])
        mkt.publishLastChanged()

        # Should see no changes except for d1, none of the other
        # objects was in 'published' state.
        info = getChanges()
        self.assertEquals(info['adds'], [])
        self.assertEquals(info['modifications'], [])
        self.assertEquals(info['removes'], [('d1',)])

        # Publish the objects
        action(mkt.f2, 'publish')
        action(mkt.f2.d21, 'publish')
        mkt.publishLastChanged()

        # Should see changes to f2 and f2/d1, which were now
        # published. d1 has already been removed by the last
        # deployment.
        info = getChanges()
        self.assertEquals(info['adds'],
                          [('f2',), ('f2', 'd21')])
        self.assertEquals(info['modifications'], [])
        self.assertEquals(info['removes'], [])

        # Renamed 'd21' to 'd21a', equals to remove 'd21' and add 'd21a'
        self.commit()
        mkt.f2.manage_renameObjects(ids=['d21'], new_ids=['d21a'])

        mkt.publishLastChanged()
        info = getChanges()
        self.assertEquals(info['adds'],
                          [('f2', 'd21a')])
        self.assertEquals(info['modifications'], [('f2',)])
        self.assertEquals(info['removes'], [('f2', 'd21',)])

        # Renamed 'f2' to 'f2a', equals to remove f2, f2/d21a and add
        # f2a and f2/d21a
        self.commit()
        mkt.manage_renameObjects(ids=['f2'], new_ids=['f2a'])

        mkt.publishLastChanged()
        info = getChanges()
        self.assertEquals(info['adds'],
                          [('f2a',), ('f2a', 'd21a')])
        self.assertEquals(info['modifications'], [])
        self.assertEquals(info['removes'], [('f2',), ('f2', 'd21a',)])

        # Make a minor modification to make sure it survives a publish.
        self.assertEquals(mkt.f1.d11.Title(), 'Document One')
        self.setTitle(mkt.f1.d11, 'Document Alpha One')
        self.assertEquals(mkt.f1.d11.Title(), 'Document Alpha One')

        # Full publish, mark everything as modified
        mkt.publishStage()
        info = getChanges()
        self.assertEquals(info['adds'], [])
        self.assertEquals(info['modifications'],
                          [('f1',), ('f1', 'd11'),
                           ('f2a',), ('f2a', 'd21a')])
        self.assertEquals(info['removes'], [])
        self.assertEquals(mkt.f1.d11.Title(), 'Document Alpha One')

        # Retract f2a/d21a, should show up as removed in next deployment.
        action(mkt.f2a.d21a, 'retract')

        # Full publish, mark everything as modified
        mkt.publishStage()
        info = getChanges()
        self.assertEquals(info['adds'], [])
        self.assertEquals(info['modifications'],
                          [('f1',), ('f1', 'd11'), ('f2a',)])
        self.assertEquals(info['removes'], [('f2a', 'd21a')])
        self.assertEquals(mkt.f1.d11.Title(), 'Document Alpha One')

class TestInPlaceChangeSets(TestChangeSets):

    def afterSetUp(self):
        TestChangeSets.afterSetUp(self)
        self.home.mkt.setInPlace(True)

class TestPloneChangeSets(TestChangeSets, TestStaging): pass

class TestZopeChangeSets(TestChangeSets, TestZopeStaging):

    def afterSetUp(self):
        TestChangeSets.afterSetUp(self)
        self.home.mkt.setStagingUseCatalogOnly(False)

class TestPloneInPlaceChangeSets(TestInPlaceChangeSets, TestStaging): pass

class TestZopeInPlaceChangeSets(TestInPlaceChangeSets, TestZopeStaging):

    def afterSetUp(self):
        TestInPlaceChangeSets.afterSetUp(self)
        self.home.mkt.setStagingUseCatalogOnly(False)

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    for testclass in (
        TestPloneChangeSets,
        TestZopeChangeSets,
        TestPloneInPlaceChangeSets,
        TestZopeInPlaceChangeSets,
        TestWFChangeSets,
        ):
        suite.addTest(makeSuite(testclass))
    return suite

if __name__ == '__main__':
    framework(descriptions=1, verbosity=1)
